@@ -47,17 +47,14 @@ module H    = ProceduralHelpers
 type status = {
    sorts : (C.id, A.sort_kind) Hashtbl.t;
    types : (C.id, A.anntypes) Hashtbl.t;
-   prefix: string;
    max_depth: int option;
    depth: int;
    context: C.context;
-   intros: string option list;
-   clears: string list;
-   clears_note: string;
-   case: int list;
-   skip_thm_and_qed : bool;
+   case: int list
 }
 
+let debug = false
+
 (* helpers ******************************************************************)
 
 let split2_last l1 l2 =
@@ -86,12 +83,9 @@ let string_of_head = function
    | C.AMeta _         -> "meta"
    | C.AImplicit _     -> "implict"
 
-let clear st = {st with intros = []}
+let next st = {st with depth = succ st.depth}
 
-let next st = {(clear st) with depth = succ st.depth}
-
-let add st entry intro =
-   {st with context = entry :: st.context; intros = intro :: st.intros}
+let add st entry = {st with context = entry :: st.context}
 
 let push st = {st with case = 1 :: st.case}
 
@@ -175,9 +169,26 @@ try
       | (_, _, _, cs) -> List.map fst cs  
 with Invalid_argument _ -> failwith "A2P.get_ind_names"
 
+let string_of_atomic = function
+   | C.ARel (_, _, _, s)               -> s
+   | C.AVar (_, uri, _)                -> H.name_of_uri uri None None
+   | C.AConst (_, uri, _)              -> H.name_of_uri uri None None
+   | C.AMutInd (_, uri, i, _)          -> H.name_of_uri uri (Some i) None
+   | C.AMutConstruct (_, uri, i, j, _) -> H.name_of_uri uri (Some i) (Some j)
+   | _                                 -> ""
+
+let get_sub_names head l =
+   let s = string_of_atomic head in
+   if s = "" then [] else
+   let map (names, i) _ = 
+      let name = Printf.sprintf "%s_%u" s i in name :: names, succ i
+   in
+   let names, _ = List.fold_left map ([], 1) l in 
+   List.rev names
+
 (* proof construction *******************************************************)
 
-let used_premise = C.Name "USED"
+let anonymous_premise = C.Name "PREMISE"
 
 let mk_exp_args hd tl classes synth =
    let meta id = C.AImplicit (id, None) in
@@ -195,24 +206,32 @@ let mk_exp_args hd tl classes synth =
 let mk_convert st ?name sty ety note =
    let e = Cn.hole "" in
    let csty, cety = H.cic sty, H.cic ety in
-   let _note = Printf.sprintf "%s\nSINTH: %s\nEXP: %s"
-      note (Pp.ppterm csty) (Pp.ppterm cety)
+   let script = 
+      if debug then
+         let sname = match name with None -> "" | Some (id, _) -> id in
+         let note = Printf.sprintf "%s: %s\nSINTH: %s\nEXP: %s"
+            note sname (Pp.ppterm csty) (Pp.ppterm cety)
+	 in 
+	 [T.Note note]
+      else []
    in
-   assert (Ut.is_sober csty); 
-   assert (Ut.is_sober cety);
-   if Ut.alpha_equivalence csty cety then [(* T.Note note *)] else 
+   assert (Ut.is_sober st.context csty); 
+   assert (Ut.is_sober st.context cety);
+   if Ut.alpha_equivalence csty cety then script else 
    let sty, ety = H.acic_bc st.context sty, H.acic_bc st.context ety in
    match name with
-      | None         -> [T.Change (sty, ety, None, e, ""(*note*))]
+      | None         -> T.Change (sty, ety, None, e, "") :: script
       | Some (id, i) -> 
          begin match get_entry st id with
-	    | C.Def _  -> assert false (* [T.ClearBody (id, note)] *)
-	    | C.Decl _ -> [T.Change (ety, sty, Some (id, Some id), e, "" (* note *))] 
+	    | C.Def _  -> assert false (* T.ClearBody (id, "") :: script *)
+	    | C.Decl _ -> 
+	       T.Change (ety, sty, Some (id, Some id), e, "") :: script 
          end
 
 let convert st ?name v = 
    match get_inner_types st v with
-      | None            -> [(*T.Note "NORMAL: NO INNER TYPES"*)]
+      | None            -> 
+         if debug then [T.Note "NORMAL: NO INNER TYPES"] else []
       | Some (sty, ety) -> mk_convert st ?name sty ety "NORMAL"
 
 let convert_elim st ?name t v pattern =
@@ -231,56 +250,61 @@ let get_intro = function
    | C.Anonymous -> None
    | C.Name s    -> Some s
 
-let mk_intros st what script =
-   let intros st script =
-      if st.intros = [] then script else
-      let count = List.length st.intros in
-      T.Intros (Some count, List.rev st.intros, "") :: script
-   in
-   let clears st script =
-      if true (* st.clears = [] *) then script else T.Clear (st.clears, st.clears_note) :: script
-   in
-   intros st (clears st (convert st what @ script))   
+let mk_preamble st what script =
+   convert st what @ script   
 
 let mk_arg st = function
    | C.ARel (_, _, i, name) as what -> convert st ~name:(name, i) what
    | _                              -> []
 
-let mk_fwd_rewrite st dtext name tl direction t =   
+let mk_fwd_rewrite st dtext name tl direction v t ity =
+   let compare premise = function
+      | None   -> true
+      | Some s -> s = premise
+   in
    assert (List.length tl = 6);
    let what, where, predicate = List.nth tl 5, List.nth tl 3, List.nth tl 2 in
    let e = Cn.mk_pattern 1 predicate in
+   if (Cn.does_not_occur e) then st, [] else 
    match where with
-      | C.ARel (_, _, i, premise) as v ->
-         let where = Some (premise, name) in
-(*         let _script = convert_elim st ~name:(premise, i) t v e in *) 
-         let script = mk_arg st what @ mk_arg st v (* @ script *) in
-	 let st = {st with context = Cn.clear st.context premise} in 
-	 st, T.Rewrite (direction, what, where, e, dtext) :: script
+      | C.ARel (_, _, i, premise) as w ->
+(*         let _script = convert_elim st ~name:(premise, i) v w e in *) 
+         let script name =
+            let where = Some (premise, name) in
+	    let script = mk_arg st what @ mk_arg st w (* @ script *) in
+	    T.Rewrite (direction, what, where, e, dtext) :: script
+	 in
+	 if DTI.does_not_occur (succ i) (H.cic t) || compare premise name then
+	    {st with context = Cn.clear st.context premise}, script name
+	 else begin
+	    assert (Ut.is_sober st.context (H.cic ity));
+	    let ity = H.acic_bc st.context ity in
+	    let br1 = [T.Id ""] in
+	    let br2 = List.rev (T.Apply (w, "assumption") :: script None) in
+	    let text = "non linear rewrite" in
+	    st, [T.Branch ([br2; br1], ""); T.Cut (name, ity, text)]
+	 end
       | _                         -> assert false
 
 let mk_rewrite st dtext where qs tl direction t = 
    assert (List.length tl = 5);
    let predicate = List.nth tl 2 in
    let e = Cn.mk_pattern 1 predicate in
-   let script = [] (* convert_elim st t t e *) in
-   script @ [T.Rewrite (direction, where, None, e, dtext); T.Branch (qs, "")]
+   let script = [T.Branch (qs, "")] in
+   if (Cn.does_not_occur e) then script else 
+(*   let script = convert_elim st t t e in *)
+   T.Rewrite (direction, where, None, e, dtext) :: script
 
-let rec proc_lambda st name v t =
-   let dno = DTI.does_not_occur 1 (H.cic t) in
-   let dno = dno && match get_inner_types st t with
-      | None          -> false
-      | Some (it, et) -> 
-         DTI.does_not_occur 1 (H.cic it) && DTI.does_not_occur 1 (H.cic et)
-   in
-   let name = match dno, name with
-      | true, _            -> C.Anonymous
-      | false, C.Anonymous -> H.mk_fresh_name st.context used_premise 
-      | false, name        -> name
+let rec proc_lambda st what name v t =
+   let name = match name with
+      | C.Anonymous -> H.mk_fresh_name st.context anonymous_premise
+      | name        -> name
    in
    let entry = Some (name, C.Decl (H.cic v)) in
    let intro = get_intro name in
-   proc_proof (add st entry intro) t
+   let script = proc_proof (add st entry) t in
+   let script = T.Intros (Some 1, [intro], "") :: script in
+   mk_preamble st what script
 
 and proc_letin st what name v w t =
    let intro = get_intro name in
@@ -290,12 +314,13 @@ and proc_letin st what name v w t =
          | Some (ity, _) ->
 	    let st, rqv = match v with
                | C.AAppl (_, hd :: tl) when is_fwd_rewrite_right hd tl ->
-	          mk_fwd_rewrite st dtext intro tl true v
+	          mk_fwd_rewrite st dtext intro tl true v t ity
 	       | C.AAppl (_, hd :: tl) when is_fwd_rewrite_left hd tl  ->
-	          mk_fwd_rewrite st dtext intro tl false v
+	          mk_fwd_rewrite st dtext intro tl false v t ity
 	       | v                                                     ->
+	          assert (Ut.is_sober st.context (H.cic ity));
+		  let ity = H.acic_bc st.context ity in
 	          let qs = [proc_proof (next st) v; [T.Id ""]] in
-	          let ity = H.acic_bc st.context ity in
 		  st, [T.Branch (qs, ""); T.Cut (intro, ity, dtext)]
 	    in
 	    st, C.Decl (H.cic ity), rqv
@@ -303,42 +328,44 @@ and proc_letin st what name v w t =
 	    st, C.Def (H.cic v, H.cic w), [T.LetIn (intro, v, dtext)]
       in
       let entry = Some (name, hyp) in
-      let qt = proc_proof (next (add st entry intro)) t in
+      let qt = proc_proof (next (add st entry)) t in
       List.rev_append rqv qt      
    else
       [T.Apply (what, dtext)]
    in
-   mk_intros st what script
+   mk_preamble st what script
 
 and proc_rel st what = 
    let _, dtext = test_depth st in
    let text = "assumption" in
    let script = [T.Apply (what, dtext ^ text)] in 
-   mk_intros st what script
+   mk_preamble st what script
 
 and proc_mutconstruct st what = 
    let _, dtext = test_depth st in
    let script = [T.Apply (what, dtext)] in 
-   mk_intros st what script
+   mk_preamble st what script
 
 and proc_const st what = 
    let _, dtext = test_depth st in
    let script = [T.Apply (what, dtext)] in 
-   mk_intros st what script
+   mk_preamble st what script
 
 and proc_appl st what hd tl =
    let proceed, dtext = test_depth st in
    let script = if proceed then
       let ty = get_type "TC2" st hd in
       let classes, rc = Cl.classify st.context ty in
-      let goal_arity = match get_inner_types st what with
-         | None          -> 0
-	 | Some (ity, _) -> snd (PEH.split_with_whd (st.context, H.cic ity))
+      let goal_arity, goal = match get_inner_types st what with
+         | None            -> 0, None
+	 | Some (ity, ety) -> 
+	   snd (PEH.split_with_whd (st.context, H.cic ity)), Some (H.cic ety)
       in
       let parsno, argsno = List.length classes, List.length tl in
       let decurry = parsno - argsno in
       let diff = goal_arity - decurry in
       if diff < 0 then failwith (Printf.sprintf "NOT TOTAL: %i %s |--- %s" diff (Pp.ppcontext st.context) (Pp.ppterm (H.cic hd)));
+      let classes = Cl.adjust st.context tl ?goal classes in
       let rec mk_synth a n =
          if n < 0 then a else mk_synth (I.S.add n a) (pred n)
       in
@@ -367,18 +394,20 @@ and proc_appl st what hd tl =
 	       (* convert_elim st what what e @ *) script2 @ 
 	       [T.Elim (where, using, e, dtext ^ text); T.Branch (qs, "")]
 	 | None        ->
-	    let qs = proc_bkd_proofs (next st) synth [] classes tl in
+	    let names = get_sub_names hd tl in
+	    let qs = proc_bkd_proofs (next st) synth names classes tl in
 	    let hd = mk_exp_args hd tl classes synth in
 	    script @ [T.Apply (hd, dtext ^ text); T.Branch (qs, "")]
    else
       [T.Apply (what, dtext)]
    in
-   mk_intros st what script
+   mk_preamble st what script
 
 and proc_other st what =
+   let _, dtext = test_depth st in
    let text = Printf.sprintf "%s: %s" "UNEXPANDED" (string_of_head what) in
-   let script = [T.Note text] in
-   mk_intros st what script
+   let script = [T.Apply (what, dtext ^ text)] in 
+   mk_preamble st what script
 
 and proc_proof st t = 
    let f st =
@@ -388,12 +417,11 @@ and proc_proof st t =
 	  (Pp.ppterm (H.cic it)) (Pp.ppterm (H.cic et))) 
          | None          -> None, "\nNo types"
       in
-      let context, clears = Cn.get_clears st.context (H.cic t) xtypes in
-      let note = Pp.ppcontext st.context ^ note in
-      {st with context = context; clears = clears; clears_note = note; }
+      let context, _clears = Cn.get_clears st.context (H.cic t) xtypes in
+      {st with context = context}
    in
    match t with
-      | C.ALambda (_, name, w, t)           -> proc_lambda st name w t
+      | C.ALambda (_, name, w, t) as what   -> proc_lambda (f st) what name w t
       | C.ALetIn (_, name, v, w, t) as what -> proc_letin (f st) what name v w t
       | C.ARel _ as what                    -> proc_rel (f st) what
       | C.AMutConstruct _ as what           -> proc_mutconstruct (f st) what
@@ -418,7 +446,7 @@ try
    let aux (inv, _) v =
       if I.overlaps synth inv then None else
       if I.S.is_empty inv then Some (get_note (fun st -> proc_proof st v)) else
-      Some (fun _ -> [T.Apply (v, dtext ^ "dependent")])
+      Some (get_note (fun _ -> [T.Apply (v, dtext ^ "dependent")]))
    in	
    let ps = T.list_map2_filter aux classes ts in
    let b = List.length ps > 1 in
@@ -433,34 +461,50 @@ let is_theorem pars =
    List.mem (`Flavour `Theorem) pars || List.mem (`Flavour `Fact) pars || 
    List.mem (`Flavour `Remark) pars || List.mem (`Flavour `Lemma) pars
 
+let is_definition pars =
+   List.mem (`Flavour `Definition) pars
+
 let proc_obj st = function
-   | C.AConstant (_, _, s, Some v, t, [], pars) when is_theorem pars ->
+   | C.AConstant (_, _, s, Some v, t, [], pars) when is_theorem pars    ->
       let ast = proc_proof st v in
       let steps, nodes = T.count_steps 0 ast, T.count_nodes 0 ast in
       let text = Printf.sprintf "tactics: %u\nnodes: %u" steps nodes in
-      if st.skip_thm_and_qed then ast
-      else T.Theorem (Some s, t, "") :: ast @ [T.Qed text]
-   | _                                                               ->
-      failwith "not a theorem"
+      T.Statement (`Theorem, Some s, t, None, "") :: ast @ [T.Qed text]
+   | C.AConstant (_, _, s, Some v, t, [], pars) when is_definition pars ->
+      [T.Statement (`Definition, Some s, t, Some v, "")]
+   | C.AConstant (_, _, s, None, t, [], pars)                           ->
+      [T.Statement (`Axiom, Some s, t, None, "")]
+   | _                                                                  ->
+      failwith "not a theorem, definition, axiom"
 
 (* interface functions ******************************************************)
 
-let acic2procedural ~ids_to_inner_sorts ~ids_to_inner_types ?depth
-?(skip_thm_and_qed=false) prefix aobj = 
+let procedural_of_acic_object ~ids_to_inner_sorts ~ids_to_inner_types ?depth
+   prefix anobj = 
    let st = {
       sorts       = ids_to_inner_sorts;
       types       = ids_to_inner_types;
-      prefix      = prefix;
       max_depth   = depth;
       depth       = 0;
       context     = [];
-      intros      = [];
-      clears      = [];
-      clears_note = "";
-      case        = [];
-      skip_thm_and_qed = skip_thm_and_qed;
+      case        = []
    } in
    HLog.debug "Procedural: level 2 transformation";
-   let steps = proc_obj st aobj in
+   let steps = proc_obj st anobj in
+   HLog.debug "Procedural: grafite rendering";
+   List.rev (T.render_steps [] steps)
+
+let procedural_of_acic_term ~ids_to_inner_sorts ~ids_to_inner_types ?depth
+   prefix context annterm = 
+   let st = {
+      sorts       = ids_to_inner_sorts;
+      types       = ids_to_inner_types;
+      max_depth   = depth;
+      depth       = 0;
+      context     = context;
+      case        = []
+   } in
+   HLog.debug "Procedural: level 2 transformation";
+   let steps = proc_proof st annterm in
    HLog.debug "Procedural: grafite rendering";
    List.rev (T.render_steps [] steps)
