@@ -27,6 +27,6 @@ type status
 
 val init: unit -> unit
 
-val make: string -> status
+val make: string -> string -> status
 
 val produce: status -> unit
