@@ -24,8 +24,12 @@
  *)
 
 let main =
-   let help = "Usage: transcript [ <package> | <conf_file> ]*" in
-   let process_package package = Engine.produce (Engine.make package) in
+   let cwd = ref Filename.current_dir_name in
+   let help = "Usage: transcript [ -C <dir> ] [ <package> | <conf_file> ]*" in
+   let help_C = " set working directory to <dir>" in
+   let set_cwd dir = cwd := dir in
+   let process_package package = Engine.produce (Engine.make !cwd package) in
    Engine.init ();
    Arg.parse [
+      ("-C", Arg.String set_cwd, help_C)
    ] process_package help
