@@ -34,7 +34,7 @@
 (*                                                                           *)
 (*****************************************************************************)
 
-(* $Id: cicUniv.ml 8560 2008-05-16 17:07:06Z tassi $ *)
+(* $Id: cicUniv.ml 8863 2008-07-23 11:41:37Z tassi $ *)
 
 (*****************************************************************************)
 (** open                                                                    **)
@@ -474,8 +474,14 @@ let add_eq u v b =
 let rank = ref MAL.empty;;
 
 let do_rank (b,_,_) =
-(*         print_ugraph ugraph; *)
-   let keys = MAL.fold (fun k _ acc -> k::acc) b [] in
+   let keys = 
+     MAL.fold 
+       (fun k v acc -> 
+          SOF.union acc (SOF.union (SOF.singleton k) 
+            (SOF.union v.eq_closure (SOF.union v.gt_closure v.ge_closure))))
+       b SOF.empty 
+   in
+   let keys = SOF.elements keys in
    let fall =
      List.fold_left 
        (fun acc u ->
@@ -483,7 +489,6 @@ let do_rank (b,_,_) =
            | [] -> 0, seen
            | x::tl when SOF.mem x seen -> aux k seen tl
            | x::tl ->
-(*                prerr_endline (String.make k '.' ^ string_of_universe x); *)
                let seen = SOF.add x seen in
                let t1, seen = aux (k+1) seen (SOF.elements (repr x b).eq_closure) in
                let t3, seen = aux (k+1) seen (SOF.elements (repr x b).gt_closure) in
@@ -499,11 +504,12 @@ let do_rank (b,_,_) =
    in
    rank := fall keys;
    let res = ref [] in
+   let resk = ref [] in
    MAL.iter 
      (fun k v -> 
-        if not (List.mem v !res) then res := v::!res;
-       prerr_endline (string_of_universe k ^ " = " ^ string_of_int v)) !rank;
-   !res 
+       if not (List.mem v !res) then res := v::!res;
+       resk := k :: !resk) !rank;
+   !res, !resk
 ;;
 
 let get_rank u = 
