@@ -72,7 +72,7 @@ val add_ge:
 val add_gt: 
   universe -> universe -> universe_graph -> universe_graph
 
-val do_rank: universe_graph -> int list
+val do_rank: universe_graph -> int list * universe list
 val get_rank: universe -> int
 
 (*
