@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: cicReduction.ml 8654 2008-05-30 19:28:41Z tassi $ *)
+(* $Id: cicReduction.ml 8821 2008-07-15 12:00:24Z tassi $ *)
 
 (* TODO unify exceptions *)
 
@@ -869,24 +869,14 @@ prerr_endline ("%%%%%%: " ^ CicPp.ppterm term' ^ " <==> " ^ CicPp.ppterm t1);
 *)
                aux test_equality_only context t1 term' ugraph
             with  CicUtil.Subst_not_found _ -> false,ugraph)
-        | (C.Sort (C.Type t1), C.Sort (C.Type t2)) 
-        | (C.Sort (C.CProp t1), C.Sort (C.CProp t2)) when test_equality_only ->
-            (try
-              true,(CicUniv.add_eq t2 t1 ugraph)
-             with CicUniv.UniverseInconsistency _ -> false,ugraph)
-        | (C.Sort (C.Type t1), C.Sort (C.Type t2)) 
-        | (C.Sort (C.CProp t1), C.Sort (C.CProp t2)) ->
-            (try
-              true,(CicUniv.add_ge t2 t1 ugraph)
-             with CicUniv.UniverseInconsistency _ -> false,ugraph)
-        | (C.Sort (C.CProp t1), C.Sort (C.Type t2)) when not test_equality_only ->
-            (try
-              true,(CicUniv.add_gt t2 t1 ugraph)
-             with CicUniv.UniverseInconsistency _ -> false,ugraph)
-        | (C.Sort (C.Type t1), C.Sort (C.CProp t2)) when not test_equality_only ->
-            (try
-              true,(CicUniv.add_ge t2 t1 ugraph)
-             with CicUniv.UniverseInconsistency _ -> false,ugraph)
+        | (C.Sort (C.CProp t1|C.Type t1), C.Sort (C.CProp t2|C.Type t2)) 
+          when test_equality_only ->
+            (try true,(CicUniv.add_eq t2 t1 ugraph)
+            with CicUniv.UniverseInconsistency _ -> false,ugraph)
+        | (C.Sort (C.CProp t1|C.Type t1), C.Sort (C.CProp t2|C.Type t2))
+          when not test_equality_only ->
+            (try true,(CicUniv.add_ge t2 t1 ugraph)
+            with CicUniv.UniverseInconsistency _ -> false,ugraph)
         | (C.Sort s1, C.Sort (C.Type _))
         | (C.Sort s1, C.Sort (C.CProp _)) -> (not test_equality_only),ugraph
         | (C.Sort s1, C.Sort s2) -> (s1 = s2),ugraph
