@@ -34,7 +34,7 @@
 (*                                                                           *)
 (*****************************************************************************)
 
-(* $Id: cicUnivUtils.ml 8432 2008-04-19 16:27:57Z tassi $ *)
+(* $Id: cicUnivUtils.ml 8798 2008-07-09 11:19:02Z tassi $ *)
 
 module C = Cic
 module H = UriManager.UriHashtbl 
@@ -79,6 +79,8 @@ let universes_of_obj uri t =
     | C.Meta (n,l1) -> C.Meta (n, List.map (HExtlib.map_option aux) l1)
     | C.Sort (C.Type i) -> add_result [i]; 
       C.Sort (C.Type (CicUniv.name_universe i uri))
+    | C.Sort (C.CProp i) -> add_result [i]; 
+      C.Sort (C.CProp (CicUniv.name_universe i uri))
     | C.Rel _ 
     | C.Sort _
     | C.Implicit _ as x -> x
