@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: coercGraph.mli 7539 2007-07-19 10:03:42Z tassi $ *)
+(* $Id: coercGraph.mli 8802 2008-07-10 08:00:30Z tassi $ *)
 
 (* This module implements the Query interface to the Coercion Graph *)
 
@@ -34,17 +34,12 @@ type coercion_search_result =
   | SomeCoercion of (Cic.metasenv * Cic.term * Cic.term) list
   | SomeCoercionToTgt of (Cic.metasenv * Cic.term * Cic.term) list
   | NoCoercion
-  | NotMetaClosed
-  | NotHandled of string Lazy.t
+  | NotHandled
 
 val look_for_coercion :
   Cic.metasenv -> Cic.substitution -> Cic.context ->
    Cic.term -> Cic.term -> coercion_search_result
 
-val look_for_coercion' :
-  Cic.metasenv -> Cic.substitution -> Cic.context ->
-   CoercDb.coerc_carr -> CoercDb.coerc_carr -> coercion_search_result
-
 (* checks if term is a constant or 
  * a constant applyed that is marked with (`Class `Coercion) *)
 val is_composite: Cic.term -> bool
@@ -54,7 +49,7 @@ val source_of: Cic.term -> Cic.term
 val generate_dot_file: unit -> string
 
 (* given the Appl contents returns the argument of the head coercion *)
-val coerced_arg: Cic.term list -> Cic.term option
+val coerced_arg: Cic.term list -> (Cic.term * int) option
 
 (* returns: (carr,menv,(saturated coercion,last arg)option,idem) list *)
 val meets : 
