@@ -32,7 +32,7 @@
 (*                                                                         *)
 (***************************************************************************)
 
-(* $Id: content2pres.ml 8611 2008-05-21 12:23:53Z tassi $ *)
+(* $Id: content2pres.ml 8835 2008-07-18 12:55:24Z tassi $ *)
 
 module P = Mpresentation
 module B = Box
@@ -815,7 +815,9 @@ let conjecture2pres term2pres (id, n, context, ty) =
   (B.b_hv [Some "helm", "xref", id]
      ((B.b_toggle [
         B.b_h [] [B.b_text [] "{...}"; B.b_space];
-        B.b_hv [] (List.map
+        B.b_hv [] (HExtlib.list_concat ~sep:[B.b_text [] ";"; B.b_space] 
+         (List.map (fun x -> [x])
+         (List.map
           (function
              | None ->
 		 B.b_h []
@@ -833,7 +835,7 @@ let conjecture2pres term2pres (id, n, context, ty) =
 			    (match dec_name with
 				 None -> "_"
 			       | Some n -> n));
-                       B.b_text [] ":";
+                       B.b_text [] ":"; B.b_space; 
                        term2pres ty ]
              | Some (`Definition d) ->
                  let
@@ -846,6 +848,7 @@ let conjecture2pres term2pres (id, n, context, ty) =
 					  None -> "_"
 					| Some n -> n)) ;
                        B.b_text [] (Utf8Macro.unicode_of_tex "\\Assign");
+                       B.b_space;
                        term2pres bo]
              | Some (`Proof p) ->
                  let proof_name = p.Content.proof_name in
@@ -855,12 +858,16 @@ let conjecture2pres term2pres (id, n, context, ty) =
 					  None -> "_"
 					| Some n -> n)) ;
                        B.b_text [] (Utf8Macro.unicode_of_tex "\\Assign");
+                       B.b_space;
                        proof2pres true term2pres p])
-          (List.rev context)) ] ::
+          (List.rev context)))) ] ::
          [ B.b_h []
-           [ B.b_text [] (Utf8Macro.unicode_of_tex "\\vdash");
+           [ B.b_space; 
+             B.b_text [] (Utf8Macro.unicode_of_tex "\\vdash");
+             B.b_space;
              B.b_object (p_mi [] (string_of_int n)) ;
              B.b_text [] ":" ;
+             B.b_space;
              term2pres ty ]])))
 
 let metasenv2pres term2pres = function
