@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteEngine.ml 8754 2008-06-24 14:47:16Z tassi $ *)
+(* $Id: grafiteEngine.ml 8795 2008-07-07 21:01:17Z tassi $ *)
 
 module PEH = ProofEngineHelpers
 
@@ -82,6 +82,7 @@ let rec tactic_of_ast status ast =
   (* First order tactics *)
   | GrafiteAst.Absurd (_, term) -> Tactics.absurd term
   | GrafiteAst.Apply (_, term) -> Tactics.apply term
+  | GrafiteAst.ApplyP (_, term) -> Tactics.applyP term
   | GrafiteAst.ApplyS (_, term, params) ->
      Tactics.applyS ~term ~params ~dbd:(LibraryDb.instance ())
        ~universe:status.GrafiteTypes.universe
@@ -454,7 +455,7 @@ type 'a eval_executable =
 type 'a eval_from_moo =
  { efm_go: GrafiteTypes.status -> string -> GrafiteTypes.status }
       
-let coercion_moo_statement_of (uri,arity, saturations) =
+let coercion_moo_statement_of (uri,arity, saturations,_) =
   GrafiteAst.Coercion
    (HExtlib.dummy_floc, CicUtil.term_of_uri uri, false, arity, saturations)
 
@@ -492,11 +493,11 @@ let eval_coercion status ~add_composites uri arity saturations =
    saturations (GrafiteTypes.get_baseuri status)
  in
  let moo_content = 
-   List.map coercion_moo_statement_of ((uri,arity,saturations)::compounds)
+   List.map coercion_moo_statement_of ((uri,arity,saturations,0)::compounds)
  in
  let status = GrafiteTypes.add_moo_content moo_content status in
   {status with GrafiteTypes.proof_status = GrafiteTypes.No_proof},
-   List.map (fun u,_,_ -> u) compounds
+   List.map (fun u,_,_,_ -> u) compounds
 
 module MatitaStatus =
  struct
@@ -607,9 +608,9 @@ let add_coercions_of_record_to_moo obj lemmas status =
               in
               let is_a_coercion, arity_coercion = is_a_coercion uri in
               if is_a_coercion then
-                Some (uri, coercion_moo_statement_of (uri,arity_coercion,0))
+                Some (uri, coercion_moo_statement_of (uri,arity_coercion,0,0))
               else if is_a_wanted_coercion then
-                Some (uri, coercion_moo_statement_of (uri,arity_wanted,0))
+                Some (uri, coercion_moo_statement_of (uri,arity_wanted,0,0))
               else
                 None)
             lemmas)
