@@ -33,8 +33,8 @@ val add_coercion:
   add_composites:bool -> GrafiteTypes.status ->
   UriManager.uri -> int -> int ->
   string (* baseuri *) ->
-    GrafiteTypes.status * (UriManager.uri * int * int) list
-     (* URI, arity, saturations *)
+    GrafiteTypes.status * (UriManager.uri * int * int * int) list
+     (* URI, arity, saturations, cpos *)
 
 val time_travel: 
   present:GrafiteTypes.status -> past:GrafiteTypes.status -> unit
