@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: dependenciesParser.ml 8052 2008-01-10 22:32:03Z tassi $ *)
+(* $Id: dependenciesParser.ml 8845 2008-07-21 18:10:30Z fguidi $ *)
 
 exception UnableToInclude of string
 
@@ -31,10 +31,12 @@ exception UnableToInclude of string
 type dependency =
   | IncludeDep of string
   | UriDep of UriManager.uri
-  
+  | InlineDep of string
+
 let pp_dependency = function
   | IncludeDep str -> "include \"" ^ str ^ "\""
   | UriDep uri -> "uri \"" ^ UriManager.string_of_uri uri ^ "\""
+  | InlineDep str -> "inline \"" ^ str ^ "\"" 
 
 let parse_dependencies lexbuf = 
   let tok_stream,_ =
@@ -56,6 +58,8 @@ let parse_dependencies lexbuf =
           true, (IncludeDep fname :: acc)
       | [< '("IDENT", "include'"); '("QSTRING", fname) >] ->
           true, (IncludeDep fname :: acc)
+      | [< '("IDENT", "inline"); '("IDENT", "procedural"); '("QSTRING", fname) >] ->
+          true, (InlineDep fname :: acc)
       | [< '("EOI", _) >] -> false, acc
       | [< 'tok >] -> true, acc
       | [<  >] -> false, acc) tok_stream
