@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: grafiteDisambiguate.ml 8754 2008-06-24 14:47:16Z tassi $ *)
+(* $Id: grafiteDisambiguate.ml 8782 2008-07-02 17:33:03Z fguidi $ *)
 
 exception BaseUriNotSetYet
 
@@ -207,6 +207,9 @@ let rec disambiguate_tactic
     | GrafiteAst.Apply (loc, term) ->
         let metasenv,cic = disambiguate_term context metasenv term in
         metasenv,GrafiteAst.Apply (loc, cic)
+    | GrafiteAst.ApplyP (loc, term) ->
+        let metasenv,cic = disambiguate_term context metasenv term in
+        metasenv,GrafiteAst.ApplyP (loc, cic)
     | GrafiteAst.ApplyS (loc, term, params) ->
         let metasenv, params = disambiguate_auto_params metasenv params in
         let metasenv,cic = disambiguate_term context metasenv term in
