@@ -118,7 +118,8 @@ let quick_connect
              else 
                Sqlite3.Data.INT 0L
          | _ -> raise (Error "wrong types to 'REGEXP'")
-        with exn -> HLog.error (Printexc.to_string exn); raise exn);
+        with Sys.Break -> Sqlite3.Data.INT 0L
+          | exn -> HLog.error (Printexc.to_string exn); raise exn);
   Some db
 ;;
 
