@@ -30,7 +30,7 @@ val process_notation: LexiconAst.command -> notation_id list
 val remove_notation: notation_id -> unit
 
 (** {2 Notation enabling/disabling}
- * Right now, only disabling of notation during pretty printing is supporting.
+ * Right now, only disabling of notation during pretty printing is supported.
  * If it is useful to disable it also for the input phase is still to be
  * understood ... *)
 
@@ -38,3 +38,10 @@ val get_all_notations: unit -> (notation_id * string) list  (* id, dsc *)
 val get_active_notations: unit -> notation_id list
 val set_active_notations: notation_id list -> unit
 
+(* resets internal couenters. this is an hack used in matitaScript.
+ * if you are in the middle of a script (with an history you may use to undo
+ * with some notations id inside) and you want to compile an external file 
+ * in an empty environment you need, after its compilation, to restore
+ * the previous environment (re-executing all notations commands) and this must
+ * produce the same ids as before, otherwise history is wrong. *)
+ val reset: unit -> unit
