@@ -51,7 +51,7 @@ val add_coercion:
   add_composites:bool -> 
   RefinementTool.kit -> UriManager.uri -> int (* arity *) ->
    int (* saturations *) -> string (* baseuri *) ->
-    (UriManager.uri * int * int) list (* URI, arity, saturations *)
+    (UriManager.uri * int * int * int) list (* URI, arity, saturations, cpos *)
 
 (* inverse of add_coercion, removes both the eventually created composite   *)
 (* coercions and the information that [uri] and the composites are coercion *)
