@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: metadataQuery.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: metadataQuery.ml 8450 2008-04-22 19:18:53Z tassi $ *)
 
 open Printf
 
@@ -51,11 +51,11 @@ let signature_of_hypothesis context metasenv =
             try 
               let ty,_ = 
                 CicTypeChecker.type_of_aux' 
-                  metasenv current_ctx t CicUniv.empty_ugraph 
+                  metasenv current_ctx t CicUniv.oblivion_ugraph 
               in
               let sort,_ = 
                 CicTypeChecker.type_of_aux' 
-                  metasenv current_ctx ty CicUniv.empty_ugraph 
+                  metasenv current_ctx ty CicUniv.oblivion_ugraph 
               in
               let set = Constr.UriManagerSet.union set(Constr.constants_of ty)in
               match sort with
@@ -113,15 +113,15 @@ let compare_goal_list proof goal1 goal2 =
   let (_, ey1, ty1) = CicUtil.lookup_meta goal1 metasenv in
   let (_, ey2, ty2) =  CicUtil.lookup_meta goal2 metasenv in
   let ty_sort1,_ = 
-    CicTypeChecker.type_of_aux' metasenv ey1 ty1 CicUniv.empty_ugraph 
+    CicTypeChecker.type_of_aux' metasenv ey1 ty1 CicUniv.oblivion_ugraph 
   in
   let ty_sort2,_ = 
-    CicTypeChecker.type_of_aux' metasenv ey2 ty2 CicUniv.empty_ugraph 
+    CicTypeChecker.type_of_aux' metasenv ey2 ty2 CicUniv.oblivion_ugraph 
   in
   let prop1 =
     let b,_ = 
       CicReduction.are_convertible 
-	ey1 (Cic.Sort Cic.Prop) ty_sort1 CicUniv.empty_ugraph 
+	ey1 (Cic.Sort Cic.Prop) ty_sort1 CicUniv.oblivion_ugraph 
     in
       if b then 0
       else 1
@@ -129,7 +129,7 @@ let compare_goal_list proof goal1 goal2 =
   let prop2 =
     let b,_ = 
       CicReduction.are_convertible 
-	ey2 (Cic.Sort Cic.Prop) ty_sort2 CicUniv.empty_ugraph 
+	ey2 (Cic.Sort Cic.Prop) ty_sort2 CicUniv.oblivion_ugraph 
     in 
       if b then 0
       else 1
@@ -150,7 +150,7 @@ let close_with_types s metasenv context =
     (fun e bag -> 
       let t = CicUtil.term_of_uri e in
       let ty, _ = 
-        CicTypeChecker.type_of_aux' metasenv context t CicUniv.empty_ugraph  
+        CicTypeChecker.type_of_aux' metasenv context t CicUniv.oblivion_ugraph  
       in
       Constr.UriManagerSet.union bag (Constr.constants_of ty)) 
     s s
@@ -162,7 +162,7 @@ let close_with_constructors s metasenv context =
       match t with
 	  Cic.MutInd (uri,_,_)  
 	| Cic.MutConstruct (uri,_,_,_) ->  
-	    (match fst (CicEnvironment.get_obj CicUniv.empty_ugraph uri) with
+	    (match fst (CicEnvironment.get_obj CicUniv.oblivion_ugraph uri) with
 		 Cic.InductiveDefinition(tl,_,_,_) ->
 		   snd
 		     (List.fold_left
@@ -215,7 +215,7 @@ let signature_of_goal ~(dbd:HSql.dbd) ((proof, goal) as _status) =
 let is_predicate u = 
     let ty, _ = 
       try CicTypeChecker.type_of_aux' [] []
-        (CicUtil.term_of_uri u) CicUniv.empty_ugraph
+        (CicUtil.term_of_uri u) CicUniv.oblivion_ugraph
       with CicTypeChecker.TypeCheckerFailure _ -> assert false
     in
     let rec check_last_pi = function
@@ -229,7 +229,7 @@ let is_predicate u =
 let only constants uri =
   prerr_endline (UriManager.string_of_uri uri);
   let t = CicUtil.term_of_uri uri in (* FIXME: write ty_of_term *)
-  let ty,_ = CicTypeChecker.type_of_aux' [] [] t CicUniv.empty_ugraph in
+  let ty,_ = CicTypeChecker.type_of_aux' [] [] t CicUniv.oblivion_ugraph in
   let consts = Constr.constants_of ty in
 (*
   prerr_endline ("XXX " ^ UriManager.string_of_uri uri);
@@ -260,7 +260,7 @@ let types_for_equality metasenv goal =
       (fun (i,acc) _ -> 	 
 	 let ty, _ = 
            CicTypeChecker.type_of_aux' 
-             metasenv context (Cic.Rel i) CicUniv.empty_ugraph in
+             metasenv context (Cic.Rel i) CicUniv.oblivion_ugraph in
 	 let newty = types_of_equality ty in
 	   (i+1,Constr.SetSet.union newty acc)) 
       (1,all) context
@@ -288,20 +288,20 @@ let universe_of_goal ~(dbd:HSql.dbd) apply_only metasenv goal =
   let uris =
     Constr.UriManagerSet.fold
       (fun u acc -> 
-         prerr_endline ("processing "^(UriManager.string_of_uri u));
+         debug_print (lazy ("processing "^(UriManager.string_of_uri u)));
          let set_for_sigmatch = 
 	   Constr.UriManagerSet.remove u all_constants_closed in
 	 if LibraryObjects.is_eq_URI (UriManager.strip_xpointer u) then
 	   (* equality has a special treatment *)
-           (prerr_endline "special treatment";
+           (debug_print (lazy "special treatment");
 	   let tfe =
 	     Constr.SetSet.elements (types_for_equality metasenv goal) 
 	   in
 	     List.fold_left 
 	       (fun acc l ->
 		  let tyl = Constr.UriManagerSet.elements l in
-                  prerr_endline ("tyl: "^(String.concat "\n" 
-			(List.map UriManager.string_of_uri tyl)));
+                  debug_print (lazy ("tyl: "^(String.concat "\n" 
+			(List.map UriManager.string_of_uri tyl))));
 		  let set_for_sigmatch =
 		    Constr.UriManagerSet.diff set_for_sigmatch l in
 		  let uris =
@@ -310,7 +310,7 @@ let universe_of_goal ~(dbd:HSql.dbd) apply_only metasenv goal =
 		    acc @ uris) 
 	       acc tfe)
 	 else
-           (prerr_endline "normal treatment";
+           (debug_print (lazy "normal treatment");
            let uris =
              sigmatch ~dbd ~facts:false ~where:`Statement 
                (Some (u,[]),set_for_sigmatch)
@@ -424,7 +424,8 @@ let experimental_hint
       | Some (main,_) -> 
           let ty, _ = 
             CicTypeChecker.type_of_aux' 
-              metasenv context (CicUtil.term_of_uri main) CicUniv.empty_ugraph
+              metasenv context (CicUtil.term_of_uri main)
+              CicUniv.oblivion_ugraph
           in
           Constr.constants_of ty
     in
