@@ -25,7 +25,7 @@
 
 (* let _profiler = <:profiler<_profiler>>;; *)
 
-(* $Id: saturation.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: saturation.ml 8206 2008-03-11 22:02:58Z sacerdot $ *)
 
 (* set to false to disable paramodulation inside auto_tac *)
 
@@ -1277,10 +1277,11 @@ let fix_proof metasenv context all_implicits p =
 	  let metasenv,s = aux metasenv n s in
 	  let metasenv,t = aux metasenv (n+1) t in
 	    metasenv,Cic.Prod(name,s,t)
-      | Cic.LetIn(name,s,t) ->
+      | Cic.LetIn(name,s,ty,t) ->
 	  let metasenv,s = aux metasenv n s in
+	  let metasenv,ty = aux metasenv n ty in
 	  let metasenv,t = aux metasenv (n+1) t in
-	    metasenv,Cic.LetIn(name,s,t)	      
+	    metasenv,Cic.LetIn(name,s,ty,t)
       | Cic.Const(uri,ens) -> 
 	  let metasenv,ens =
 	    List.fold_right 
