@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: utils.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: utils.ml 8206 2008-03-11 22:02:58Z sacerdot $ *)
 
 let time = true;;
 let debug = false;;
@@ -102,8 +102,8 @@ let metas_of_term term =
 	TermSet.union (aux s) (aux t)
     | C.Prod(n,s,t) ->
 	TermSet.union (aux s) (aux t)
-    | C.LetIn(n,s,t) ->
-	TermSet.union (aux s) (aux t)
+    | C.LetIn(n,s,ty,t) ->
+	TermSet.union (aux s) (TermSet.union (aux ty) (aux t))
     | t -> TermSet.empty (* TODO: maybe add other cases? *)
   in
   aux term
@@ -265,7 +265,7 @@ let weight_of_term ?(consider_metas=true) ?(count_metas_occurrences=false) term
     | C.Cast (t1, t2)
     | C.Lambda (_, t1, t2)
     | C.Prod (_, t1, t2)
-    | C.LetIn (_, t1, t2) ->
+    | C.LetIn (_, t1, _, t2) ->
         let w1 = aux t1 in
         let w2 = aux t2 in
         w1 + w2 + 1
