@@ -23,7 +23,7 @@
  * http://cs.unibo.it/helm/.
  *)
 
-(* $Id: ring.ml 7922 2007-11-25 13:12:25Z tassi $ *)
+(* $Id: ring.ml 8450 2008-04-22 19:18:53Z tassi $ *)
 
 open CicReduction
 open PrimitiveTactics
@@ -503,7 +503,7 @@ let ring_tac status =
               let status' = (* status after 1st elim_type use *)
                 let context = context_of_status status in
 		let b,_ = (*TASSI : FIXME*)
-		  are_convertible context t1'' t1 CicUniv.empty_ugraph in 
+		  are_convertible context t1'' t1 CicUniv.oblivion_ugraph in 
                 if not b then begin
                   warn (lazy "t1'' and t1 are NOT CONVERTIBLE");
                   let newstatus =
@@ -544,7 +544,8 @@ let ring_tac status =
                       let status' =
                         let context = context_of_status status in
 			let b,_ = (* TASSI:FIXME *)
-			  are_convertible context t2'' t2 CicUniv.empty_ugraph 
+			  are_convertible context t2'' t2
+                          CicUniv.oblivion_ugraph 
 			in
 			  if not b then begin 
                           warn (lazy "t2'' and t2 are NOT CONVERTIBLE");
