@@ -33,7 +33,7 @@
 (*                                                                         *)
 (***************************************************************************)
 
-(* $Id: applyTransformation.ml 8147 2008-02-19 13:00:29Z fguidi $ *)
+(* $Id: applyTransformation.ml 8829 2008-07-16 18:16:19Z fguidi $ *)
 
 module G = GrafiteAst
 
@@ -198,9 +198,9 @@ let txt_of_cic_object
         let aux = GrafiteAstPp.pp_statement
          ~map_unicode_to_tex ~term_pp ~lazy_term_pp ~obj_pp in
         let script = 
-           Acic2Procedural.acic2procedural 
+           Acic2Procedural.procedural_of_acic_object 
               ~ids_to_inner_sorts ~ids_to_inner_types 
-	      ?depth ?skip_thm_and_qed prefix aobj 
+	      ?depth prefix aobj 
   in
         "\n\n" ^ String.concat "" (List.map aux script)
 
@@ -233,8 +233,27 @@ let txt_of_inline_macro ~map_unicode_to_tex style name prefix =
       let _, baseuri, _, _ = 
          Librarian.baseuri_of_script ~include_paths name
       in
-      baseuri
+      baseuri ^ "/"
    in
    txt_of_inline_uri ~map_unicode_to_tex style suri prefix
-   
-    
+
+(****************************************************************************)
+(* procedural_txt_of_cic_term *)
+
+let procedural_txt_of_cic_term ~map_unicode_to_tex n ?depth context term =
+  let annterm, ids_to_inner_sorts, ids_to_inner_types = 
+     try Cic2acic.acic_term_of_cic_term context term
+     with e -> 
+        let msg = "procedural_txt_of_cic_term: " ^ Printexc.to_string e in
+        failwith msg
+  in
+  let term_pp = term2pres ~map_unicode_to_tex (n - 8) ids_to_inner_sorts in
+  let lazy_term_pp = term_pp in
+  let obj_pp = CicNotationPp.pp_obj term_pp in
+  let aux = GrafiteAstPp.pp_statement
+     ~map_unicode_to_tex ~term_pp ~lazy_term_pp ~obj_pp in
+  let script = 
+     Acic2Procedural.procedural_of_acic_term 
+        ~ids_to_inner_sorts ~ids_to_inner_types ?depth "" context annterm 
+  in
+  String.concat "" (List.map aux script)
