@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matita.ml 8052 2008-01-10 22:32:03Z tassi $ *)
+(* $Id: matita.ml 8843 2008-07-21 11:47:15Z tassi $ *)
 
 open Printf
 
@@ -105,7 +105,9 @@ let _ =
 
   (** {{{ Debugging *)
 let _ =
-  if BuildTimeConf.debug then begin
+  if BuildTimeConf.debug ||
+     Helm_registry.get_bool "matita.debug_menu" 
+  then begin
     gui#main#debugMenu#misc#show ();
     let addDebugItem ~label callback =
       let item =
@@ -255,11 +257,11 @@ let _ =
           HLog.debug
            ((String.concat ","
               (List.map
-                (fun u,saturations ->
+                (fun u,saturations,_ ->
                   UriManager.name_of_uri u ^
                    "(" ^ string_of_int saturations ^ ")")
                 ul)) ^ ":"
-             ^ CoercDb.name_of_carr s ^ " -> " ^ CoercDb.name_of_carr t))
+             ^ CoercDb.string_of_carr s ^ " -> " ^ CoercDb.string_of_carr t))
         (CoercDb.to_list ()));
     addDebugSeparator ();
     let mview () = (MatitaMathView.sequentsViewer_instance ())#cicMathView in
