@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaGui.ml 8748 2008-06-23 13:49:50Z tassi $ *)
+(* $Id: matitaGui.ml 8868 2008-07-23 21:05:06Z tassi $ *)
 
 open Printf
 
@@ -42,7 +42,7 @@ class type browserWin =
    * lablgladecc :-(((( *)
 object
   inherit MatitaGeneratedGui.browserWin
-  method browserUri: GEdit.combo_box_entry
+  method browserUri: GEdit.entry
 end
 
 class console ~(buffer: GText.buffer) () =
@@ -907,31 +907,12 @@ class gui () =
         (fun _ -> MatitaAutoGui.auto_dialog Auto.get_auto_status);
       connect_menu_item main#showTermGrammarMenuItem 
         (fun _ -> 
-           let w = GWindow.window ~resizable:true 
-            ~position:`CENTER_ON_PARENT
-            ~title:"Terms grammar" ~width:640 ~height:400 () in
-           w#set_transient_for (main#toplevel#as_window);
-           let s = GBin.scrolled_window () in
-           (w :> GContainer.container)#add (s :> GObj.widget);
-           let t = GText.view () in
-           t#buffer#insert (Print_grammar.ebnf_of_term ()); 
-           s#add (t:>GObj.widget);
-           w#show ());
+          let c = MatitaMathView.cicBrowser () in
+          c#load (`About `Grammar));
       connect_menu_item main#showUnicodeTable
         (fun _ -> 
-           let w = GWindow.window ~resizable:true
-            ~position:`CENTER_ON_PARENT
-            ~title:"Tex/UTF8 table" ~width:640 ~height:400 () in
-           w#set_transient_for (main#toplevel#as_window);
-           let s = GBin.scrolled_window () in
-           (w :> GContainer.container)#add (s :> GObj.widget);
-           let t = GTree.view () in 
-           let m = new MatitaGtkMisc.multiStringListModel ~cols:2 t in
-           List.iter (fun (k,vs) -> 
-             m#easy_mappend [k;String.concat " " vs])
-             (Utf8Macro.pp_table ()); 
-           s#add (t:>GObj.widget);
-           w#show ());
+          let c = MatitaMathView.cicBrowser () in
+          c#load (`About `TeX));
          (* script monospace font stuff *)  
       self#updateFontSize ();
         (* debug menu *)
@@ -981,7 +962,7 @@ class gui () =
            end));
       (* math view handling *)
       connect_menu_item main#newCicBrowserMenuItem (fun () ->
-        ignore (MatitaMathView.cicBrowser ()));
+        ignore(MatitaMathView.cicBrowser ()));
       connect_menu_item main#increaseFontSizeMenuItem (fun () ->
         self#increaseFontSize ();
         MatitaMathView.increase_font_size ();
@@ -1180,12 +1161,13 @@ class gui () =
     method newBrowserWin () =
       object (self)
         inherit browserWin ()
-        val combo = GEdit.combo_box_entry ()
+        val combo = GEdit.entry ()
         initializer
           self#check_widgets ();
           let combo_widget = combo#coerce in
           uriHBox#pack ~from:`END ~fill:true ~expand:true combo_widget;
-          combo#entry#misc#grab_focus ()
+          self#toplevel#set_transient_for main#toplevel#as_window;
+          combo#misc#grab_focus ()
         method browserUri = combo
       end
 
