@@ -37,7 +37,7 @@ end
 class type browserWin =
 object
   inherit MatitaGeneratedGui.browserWin
-  method browserUri: GEdit.combo_box_entry
+  method browserUri: GEdit.entry
 end
 
 class type gui =
@@ -145,5 +145,6 @@ object
   (* method loadList: string list -> MatitaTypes.mathViewer_entry -> unit *)
   method loadInput: string -> unit
   method mathView: clickableMathView
+  method win: browserWin
 end
 
