@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitaTypes.ml 8052 2008-01-10 22:32:03Z tassi $ *)
+(* $Id: matitaTypes.ml 8868 2008-07-23 21:05:06Z tassi $ *)
 
 open Printf
 open GrafiteTypes
@@ -37,6 +37,8 @@ type abouts =
   | `Us
   | `Coercions
   | `CoercionsFull
+  | `TeX
+  | `Grammar
   ]
   
 type mathViewer_entry =
@@ -48,6 +50,7 @@ type mathViewer_entry =
   | `Metadata of [ `Deps of [`Fwd | `Back] * UriManager.uri ]
   | `Uri of UriManager.uri (* cic object uri *)
   | `Whelp of string * UriManager.uri list (* query and results *)
+  | `Univs of UriManager.uri
   ]
 
 let string_of_entry = function
@@ -56,6 +59,8 @@ let string_of_entry = function
   | `About `Us -> "about:us"
   | `About `Coercions -> "about:coercions?tred=true"
   | `About `CoercionsFull -> "about:coercions"
+  | `About `TeX -> "about:tex"
+  | `About `Grammar -> "about:grammar"
   | `Check _ -> "check:"
   | `Cic (_, _) -> "term:"
   | `Dir uri -> uri
@@ -72,6 +77,7 @@ let string_of_entry = function
           (match dir with | `Fwd -> "forward" | `Back -> "backward") ^ suri)
   | `Uri uri -> UriManager.string_of_uri uri
   | `Whelp (query, _) -> query
+  | `Univs uri -> "univs:" ^ UriManager.string_of_uri uri
 
 let entry_of_string = function
   | "about:blank" -> `About `Blank
@@ -79,6 +85,8 @@ let entry_of_string = function
   | "about:us"    -> `About `Us
   | "about:coercions?tred=true"    -> `About `Coercions
   | "about:coercions"    -> `About `CoercionsFull
+  | "about:tex"    -> `About `TeX
+  | "about:grammar"    -> `About `Grammar
   | _ ->  (* only about entries supported ATM *)
       raise (Invalid_argument "entry_of_string")
 
