@@ -25,7 +25,8 @@
 
 exception Cancel
 
-type abouts = [ `Blank | `Current_proof | `Us | `Coercions | `CoercionsFull]
+type abouts = [ `Blank | `Current_proof | `Us | `Coercions 
+ | `CoercionsFull | `TeX | `Grammar]
 
 type mathViewer_entry =
   [ `About of abouts
@@ -35,7 +36,9 @@ type mathViewer_entry =
   | `HBugs of [ `Tutors ]
   | `Metadata of [ `Deps of [`Fwd | `Back] * UriManager.uri ]
   | `Uri of UriManager.uri
-  | `Whelp of string * UriManager.uri list ]
+  | `Whelp of string * UriManager.uri list 
+  | `Univs of UriManager.uri
+  ]
 
 val string_of_entry : mathViewer_entry -> string
 val entry_of_string : string -> mathViewer_entry
