@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitac.ml 8086 2008-01-21 17:35:11Z sacerdot $ *)
+(* $Id: matitac.ml 8779 2008-07-02 12:50:18Z tassi $ *)
 
 (* compiler ala pascal/java using make *)
 let main_compiler () =
@@ -58,9 +58,9 @@ let main_compiler () =
   (* here we go *)
   if not (Helm_registry.get_bool "matita.verbose") then MatitaMisc.shutup ();
   if MatitacLib.Make.make root target then 
-    HLog.message "Compilation successful"
+    (HLog.message "Compilation successful"; 0)
   else
-    HLog.message "Compilation failed"
+    (HLog.message "Compilation failed"; 1)
 ;;
 
 let main () =
@@ -69,7 +69,7 @@ let main () =
   if      Pcre.pmatch ~pat:"^matitadep"    bin then Matitadep.main ()
   else if Pcre.pmatch ~pat:"^matitaclean"  bin then Matitaclean.main ()
   else if Pcre.pmatch ~pat:"^matitawiki"   bin then MatitaWiki.main ()
-  else main_compiler ()
+  else exit (main_compiler ())
 ;;
 
 let _ = main ()
