@@ -23,7 +23,7 @@
  * http://helm.cs.unibo.it/
  *)
 
-(* $Id: matitadep.ml 8165 2008-02-23 21:58:11Z fguidi $ *)
+(* $Id: matitadep.ml 8856 2008-07-22 17:28:07Z fguidi $ *)
 
 module GA = GrafiteAst 
 module U = UriManager
@@ -40,7 +40,8 @@ let fix_name f =
       else f
    in 
    HExtlib.normalize_path f
- 
+
+(* FG: old function left for reference *)
 let exclude excluded_files files =
    let map file = not (List.mem (fix_name file) excluded_files) in
    List.filter map files
@@ -48,7 +49,6 @@ let exclude excluded_files files =
 let main () =
 (*  let _ = print_times "inizio" in *)
   let include_paths = ref [] in
-  let excluded_files = ref [] in
   let use_stdout = ref false in
   (* all are maps from "file" to "something" *)
   let include_deps = Hashtbl.create 13 in
@@ -57,16 +57,20 @@ let main () =
   let dot_name = "depends" in 
   let dot_file = ref "" in
   let set_dot_file () = dot_file := dot_name^".dot" in
-  let set_excluded_file name = excluded_files := name :: !excluded_files in 
   (* helpers *)
-  let baseuri_of_script s = 
+  let rec baseuri_of_script s = 
      try Hashtbl.find baseuri_of s 
      with Not_found -> 
-       let _,b,_,_ = 
+       let _,b,_,_ =  
          Librarian.baseuri_of_script ~include_paths:!include_paths s 
        in
        Hashtbl.add baseuri_of s b; 
        Hashtbl.add baseuri_of_inv b s; 
+       let _ =
+          if Filename.check_suffix s ".mma" then
+	     let generated = Filename.chop_suffix s ".mma" ^ ".ma" in
+	     ignore (baseuri_of_script generated)
+       in
        b
   in
   let script_of_baseuri ma b =
@@ -86,8 +90,6 @@ let main () =
   MatitaInit.add_cmdline_spec 
     ["-dot", Arg.Unit set_dot_file,
         "Save dependency graph in dot format and generate a png";
-     "-exclude", Arg.String set_excluded_file,
-        "Exclude a file from the dependences";
      "-stdout", Arg.Set use_stdout,
         "Print dependences on stdout"
     ];
@@ -95,8 +97,11 @@ let main () =
   MatitaInit.initialize_environment ();
   if not (Helm_registry.get_bool "matita.verbose") then MatitaMisc.shutup ();
   let cmdline_args = HR.get_list HR.string "matita.args" in
+  let test x =
+     Filename.check_suffix x ".ma" || Filename.check_suffix x ".mma"
+  in
   let files = fun () -> match cmdline_args with
-     | [] -> HExtlib.find ~test:(fun x -> Filename.check_suffix x ".ma") "."
+     | [] -> HExtlib.find ~test "."
      | _  -> cmdline_args
   in
   let args = 
@@ -119,7 +124,6 @@ let main () =
          prerr_endline ("\nEnter one of these directories and retry");
          exit 1
   in
-  let args = exclude !excluded_files args in
   let ma_files = args in
   (* here we go *)
   (* fills:
@@ -129,25 +133,43 @@ let main () =
 (*  let _ = print_times "prima di iter1" in *)
   List.iter (fun ma_file -> ignore (baseuri_of_script ma_file)) ma_files;
 (*  let _ = print_times "in mezzo alle due iter" in *)
+  let map s _ l = s :: l in
+  let ma_files = Hashtbl.fold map baseuri_of [] in
   List.iter
    (fun ma_file -> 
+      let _ = if Filename.check_suffix ma_file ".mma" then
+         let generated = Filename.chop_suffix ma_file ".mma" ^ ".ma" in
+         Hashtbl.add include_deps generated ma_file;
+      in
       let ma_baseuri = baseuri_of_script ma_file in
-      let dependencies = DependenciesParser.deps_of_file ma_file in
+      let dependencies = 
+         try DependenciesParser.deps_of_file ma_file
+	 with Sys_error _ -> []
+      in
+      let handle_uri uri =
+         if not (Http_getter_storage.is_legacy uri) then
+         let dep = resolve uri ma_baseuri in
+         match dep with 
+            | None   -> ()
+            | Some u -> 
+                 match script_of_baseuri ma_file u with
+                      | Some d -> Hashtbl.add include_deps ma_file d
+                      | None   -> ()
+      in
+      let handle_script path =
+         ignore (baseuri_of_script path);
+         Hashtbl.add include_deps ma_file path
+      in
       List.iter 
        (function
-         | DependenciesParser.UriDep uri -> 
+         | DependenciesParser.UriDep uri      ->
             let uri = UriManager.string_of_uri uri in
-            if not (Http_getter_storage.is_legacy uri) then
-              let dep = resolve uri ma_baseuri in
-              (match dep with 
-              | None -> ()
-              | Some u -> 
-                  match script_of_baseuri ma_file u with
-                  | Some d -> Hashtbl.add include_deps ma_file d
-                  | None -> ())                
-         | DependenciesParser.IncludeDep path -> 
-                ignore (baseuri_of_script path);
-                Hashtbl.add include_deps ma_file path)
+	    handle_uri uri 
+         | DependenciesParser.InlineDep path  ->
+	    if Librarian.is_uri path
+	    then handle_uri path else handle_script path
+	 | DependenciesParser.IncludeDep path ->
+	    handle_script path) 
        dependencies)
    ma_files;
   (* generate regular depend output *)
@@ -196,5 +218,3 @@ let main () =
       HLog.message ("Type 'eog "^dot_name^".png' to view the graph"); 
     end;
 (*    let _ = print_times "fine" in () *)
-
-
